library(readxl)
library(data.table)

source("Code/name_cleaning_code.R")

test_run = FALSE

#Excel names & types 
name_list = list(
  data.table(
    Names = c("Hospital", "FY", "GrossPatientRev", "TotalOperatExpense", "CostToChargeRatioInpat", "InpatNongovUtil", "OutpatNongovUtil", "VolumeThreshold", "FyEnd", "CBMSubmissionDate", "BudgetedMMUCGRP"),
    Types = c("text", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "date", "date", "numeric")
  ),
  data.table(
    Names = c("Hospital", "FY", "Payor", "Date", "SeparateFlag", "InpatDiscountPerc", "OutpatDiscountPerc", "DisallowedFlag", "Notes"),
    Types = c("text", "numeric", "text", "text", "text", "numeric", "numeric", "text", "text")
  ),
  data.table(
    Names = c("Hospital", "FY", "Payor", "ContractDate", "ExpirationDate", "ContractSubmissionDate", "Discharges", "GrossRevenueInpat", "DiscountPercInpat", "DiscountAmountInpat", "NetRevenueInpat", "CostInpat", "ChargePerDischarge", "CostPerDischarge", "CostToChargeRatioInpat"),
    Types = c("text", "numeric", "text", "date", "date", "date", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric")
  ),
  data.table(
    Names = c("Hospital", "FY", "Payor", "Visits", "GrossRevenueOutpat", "DiscountPercOutpat", "DiscountAmountOutpat", "NetRevenueOutpat", "CostOutpat", "ChargePerVisit", "CostPerVisit", "CostToChargeRatioOutpat"),
    Types = c("text", "numeric", "text", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric")
  )
)

annual_dt     = as.data.table(read_xlsx("Transcriptions/CBM-B-DC Transcription.xlsx", sheet = 1, col_names = name_list[[1]]$Names, col_types = name_list[[1]]$Types, skip = 1))
contract_dt   = as.data.table(read_xlsx("Transcriptions/CBM-B-DC Transcription.xlsx", sheet = 2, col_names = name_list[[2]]$Names, col_types = name_list[[2]]$Types, skip = 1, na = "N/A"))
#Suppress warnings to allow reading expiration column as date despite auto-renew values 
detail_inpat  = suppressWarnings(as.data.table(read_xlsx("Transcriptions/CBM-B-DC Transcription.xlsx", sheet = 3, col_names = name_list[[3]]$Names, col_types = name_list[[3]]$Types, skip = 1)))
detail_inpat$ExpirationDate = trimws(ifelse(is.na(detail_inpat$ExpirationDate), as.character(as.data.table(read_xlsx("Transcriptions/CBM-B-DC Transcription.xlsx", sheet = 3))[, get("Date Expires")]), as.character(detail_inpat$ExpirationDate)))
detail_inpat$ExpirationDate = ifelse(tolower(gsub(" ", "", gsub("-", "", detail_inpat$ExpirationDate))) %in% c('auto', 'annual', 'autorenew', 'autorenewal', 'ongoing', 'monthlyrenewals'), 'Auto', detail_inpat$ExpirationDate)
contract_dt$Date            = ifelse(tolower(gsub(" ", "", gsub("-", "",            contract_dt$Date))) %in% c('auto', 'annual', 'autorenew', 'autorenewal', 'ongoing', 'monthlyrenewals'), 'Auto',            contract_dt$Date)
detail_outpat = as.data.table(read_xlsx("Transcriptions/CBM-B-DC Transcription.xlsx", sheet = 4, col_names = name_list[[4]]$Names, col_types = name_list[[4]]$Types, skip = 1))

contract_dt$Date = ifelse(is.na(contract_dt$Date), "", ifelse(contract_dt$Date == "Auto", "Auto", as.character(as.Date(as.numeric(contract_dt$Date), origin = "1899-12-30"))))
contract_dt = contract_dt[Payor != "Ignore"]

#================================
# Flags for irreconcilable data
#   Will use contract lists rather than summaries 
#================================

annual_dt[,     Flag_Unreliable := F]
contract_dt[,   Flag_Unreliable := F]
detail_inpat[,  Flag_Unreliable := F]
detail_outpat[, Flag_Unreliable := F]


detail_inpat[ Hospital == "City Hospital" & FY == 2015, Flag_Unreliable := T]
detail_outpat[Hospital == "City Hospital" & FY == 2015, Flag_Unreliable := T]

detail_inpat[ Hospital == "CAMC" & FY == 2016, Flag_Unreliable := T]
detail_outpat[Hospital == "CAMC" & FY == 2016, Flag_Unreliable := T]

#Can't get numbers to add up (missing contract?)
detail_outpat[Hospital == "Wheeling Hospital" & FY == 2011, Flag_Unreliable := T]

#Missing contracts
contract_dt[  Hospital == "City Hospital" & FY == 2014, Flag_Unreliable := T]
detail_inpat[ Hospital == "City Hospital" & FY == 2014, Flag_Unreliable := T]
detail_outpat[Hospital == "City Hospital" & FY == 2014, Flag_Unreliable := T]
detail_inpat[ Hospital == "Cabell Huntington" & FY == 2011, Flag_Unreliable := T]
detail_outpat[Hospital == "Cabell Huntington" & FY == 2011, Flag_Unreliable := T]

#Contract rate typos (n.b. DC unchanged in this period, so likely typo in DC discounts)
contract_dt[  Hospital == "CAMC Teays" & FY == 2011, Flag_Unreliable := T]
detail_inpat[ Hospital == "CAMC Teays" & FY == 2011, Flag_Unreliable := T]
detail_outpat[Hospital == "CAMC Teays" & FY == 2011, Flag_Unreliable := T]

#Discount misalignment
contract_dt[  Hospital == "Beckley Appalachian" & FY == 2016, Flag_Unreliable := T]

#Nothing adds up 
detail_inpat[ Hospital == "Charleston Area" & FY == 2012, Flag_Unreliable := T]
detail_outpat[Hospital == "Charleston Area" & FY == 2012, Flag_Unreliable := T]

#Contracts don't add up 
detail_outpat[Hospital == "West Virginia University Hospitals" & FY == 2012, Flag_Unreliable := T]
detail_inpat[ Hospital == "West Virginia University Hospitals" & FY == 2013, Flag_Unreliable := T]
detail_inpat[ Hospital == "City Hospital" & FY == 2013, Flag_Unreliable := T]
detail_outpat[Hospital == "City Hospital" & FY == 2013, Flag_Unreliable := T]

#================================
# Manual Fixes
#================================

#Apparent typos
annual_dt[, FyEnd := as.Date(FyEnd)]
annual_dt[, CBMSubmissionDate := as.Date(CBMSubmissionDate)]

annual_dt[Hospital == "Princeton Community"  & FY == 2015, FyEnd := as.Date("2015-06-30")]
annual_dt[Hospital == "Princeton Community"  & FY == 2016, FyEnd := as.Date("2016-06-30")]
annual_dt[Hospital == "CAMC" & FY == 2008, VolumeThreshold := 14758]
detail_inpat[Hospital == "Stonewall Jackson" & FY == 2016 & Payor == "Combined Contracts", GrossRevenueInpat := 1190347]
contract_dt[ Hospital == "Wheeling" & FY == 2015 & Payor == "BCBS - Wheeling Pitt", InpatDiscountPerc := 47.21]
contract_dt[ Hospital == "Wheeling" & FY == 2015 & Payor == "Cigna",               OutpatDiscountPerc := 29.95]
contract_dt[ Hospital == "Wheeling" & FY == 2014 & Payor == "Health Plan of UOV",  OutpatDiscountPerc := 48.89]

#Otherwise constant discounts nearby --- likely typo 
contract_dt[ Hospital == "West Virginia University Hospitals" & FY == 2012 & Payor == "Interplan Health (formerly DCA)", InpatDiscountPerc := 5]
#Seems like rounding in even-numbered years 
contract_dt[ Hospital == "West Virginia University Hospitals" & FY %in% c(2011, 2013) & Payor == "Aetna", OutpatDiscountPerc := 12]


detail_inpat[ Hospital == "City Hospital" & FY == 2014 & Payor %in% c("Total", "Combined Contracts"), GrossRevenueInpat  := GrossRevenueInpat  - 700000]
detail_outpat[Hospital == "City Hospital" & FY == 2014 & Payor %in% c("Total", "Combined Contracts"), GrossRevenueOutpat := GrossRevenueOutpat - 950000]

detail_inpat[Hospital == "Ohio Valley Medical Center" & FY == 2014 & Payor == "Total", ':='(
  GrossRevenueInpat   = 32442101,
  DiscountAmountInpat = 17923723,
  NetRevenueInpat     = 14518378,
  CostInpat           = 11546143
)]

detail_outpat[Hospital == "Ohio Valley Medical Center" & FY == 2014 & Payor == "Total", ':='(
  Visits               = 68763,
  GrossRevenueOutpat   = 51266254,
  DiscountAmountOutpat = 24690102,
  NetRevenueOutpat     = 26576153,
  CostOutpat           = 18245660
)]

detail_inpat[Hospital == "Princeton Community" & FY == 2014 & Payor == "Total", ':='(
  NetRevenueInpat     = NetRevenueInpat     - 2370,
  DiscountAmountInpat = DiscountAmountInpat + 2370
)]


#Total dropped a hospital
detail_inpat[Hospital %like% "Princeton" & FY == 2016 & Payor == "Total", ':='(
  Discharges          = Discharges          + 1,
  GrossRevenueInpat   = GrossRevenueInpat   + 13165.16,
  DiscountAmountInpat = DiscountAmountInpat + 5792.67,
  NetRevenueInpat     = NetRevenueInpat     + 7372.49,
  CostInpat           = CostInpat           + 4564.36
)]
detail_outpat[Hospital %like% "Princeton" & FY == 2016 & Payor == "Total", ':='(
  Visits               = Visits               + 1,
  GrossRevenueOutpat   = GrossRevenueOutpat   + 935.34,
  DiscountAmountOutpat = DiscountAmountOutpat + 224.48,
  NetRevenueOutpat     = NetRevenueOutpat     + 710.86,
  CostOutpat           = CostOutpat           + 324.28
)]
detail_inpat[Hospital %like% "Wheeling" & FY == 2015 & Payor == "Total", ':='(
  Discharges          = Discharges          + 67,
  GrossRevenueInpat   = GrossRevenueInpat   + 892668,
  DiscountAmountInpat = DiscountAmountInpat + 479965,
  NetRevenueInpat     = NetRevenueInpat     + 412703,
  CostInpat           = CostInpat           + 359691
)]
detail_outpat[Hospital %like% "Wheeling" & FY == 2015 & Payor == "Total", ':='(
  Visits               = Visits               + 2923,
  GrossRevenueOutpat   = GrossRevenueOutpat   + 2732811,
  DiscountAmountOutpat = DiscountAmountOutpat + 928980,
  NetRevenueOutpat     = NetRevenueOutpat     + 1803831,
  CostOutpat           = CostOutpat           + 1101157
)]

#There are hand-written corrections which seem unreliable & small
detail_inpat[Hospital %like% "Ohio Valley" & FY == 2015 & Payor == "Total", ':='(
  DiscountAmountInpat = DiscountAmountInpat + 172,
  NetRevenueInpat     = NetRevenueInpat     - 172,
  CostInpat           = CostInpat           + 80
)]
detail_outpat[Hospital %like% "Ohio Valley" & FY == 2015 & Payor == "Total", ':='(
  CostOutpat = CostOutpat + 363
)]

detail_inpat[ Hospital == "Wetzel County" & FY == 2011 & Payor == "Total", NetRevenueInpat := 1788212]
detail_inpat[ Hospital == "Wetzel County" & FY == 2011 & Payor == "Total", CostInpat := CostInpat + 30368]
detail_inpat[ Hospital == "Wheeling Hospital" & Payor == "Blue Cross-Wheeling Pitt" & FY == 2011, DiscountPercInpat := 57.14]
contract_dt[  Hospital == "Wheeling Hospital" & Payor == "Blue Cross-Wheeling Pitt" & FY == 2011, InpatDiscountPerc := 57.14]
detail_inpat[ Hospital == "West Virginia University Hospitals" & FY == 2011 & Payor == "Mental Health Associates", ':='(ChargePerDischarge = GrossRevenueInpat / Discharges, CostPerDischarge = CostInpat / Discharges)]

detail_outpat[Hospital == "Charleston Area" & FY == 2011 & Payor == "Total", ':='(GrossRevenueOutpat = GrossRevenueOutpat - 550, NetRevenueOutpat = NetRevenueOutpat - 550, CostOutpat = CostOutpat - 225)]

detail_outpat[Hospital == "Fairmont General"  & FY == 2011 & Payor %in% c("Total", "Combined Contracts"), NetRevenueOutpat := NetRevenueOutpat - 4101150]

detail_inpat[Hospital == "Charleston Area Medical Center" & FY == 2013 & Payor == "Total",    NetRevenueInpat := NetRevenueInpat - 10000]

#To align with other years 
detail_inpat[Hospital == "Charleston Area Medical Center" & FY == 2013 & Payor == "Carelink", ExpirationDate  := "2012-12-31"]

#Rate miscalculations
detail_inpat[Hospital == "Pleasant Valley"   & FY == 2016 & Payor %like% "Choice Care", ':='(ChargePerDischarge = round(GrossRevenueInpat / Discharges), CostPerDischarge = round(CostInpat / Discharges))]
detail_inpat[Hospital == "City Hospital"     & FY == 2015 & Payor %like% "Cigna" & Payor %like% "Psych", ':='(ChargePerDischarge = round(GrossRevenueInpat / Discharges), CostPerDischarge = round(CostInpat / Discharges))]
detail_inpat[Hospital == "City Hospital"     & FY == 2015 & Payor %like% "Health Plan IPN", ':='(ChargePerDischarge = round(GrossRevenueInpat / Discharges), CostPerDischarge = round(CostInpat / Discharges))]
detail_inpat[Hospital == "Camden Clark"      & FY == 2014 & Payor %like% "Blue Cross PPO", ':='(ChargePerDischarge = round(GrossRevenueInpat / Discharges))]
detail_inpat[Hospital == "Davis Medical"     & FY == 2016 & Payor == "Blue Cross",   CostPerDischarge := round(CostInpat  / Discharges)]
detail_outpat[Hospital== "Davis Medical"     & FY == 2016 & Payor == "Blue Cross",   CostPerVisit     := round(CostOutpat / Visits)]
detail_outpat[Hospital %like% "Princeton"    & FY == 2015 & Payor == "Blue Cross",   ChargePerVisit     := round(GrossRevenueOutpat / Visits)]
detail_outpat[Hospital == "Weirton"          & FY == 2016 & Payor %like% "Combined", DiscountPercOutpat := 100 * round(DiscountAmountOutpat / GrossRevenueOutpat, 4)]
detail_inpat[ Hospital == "CAMC Teays"       & FY == 2013 & Payor == "United et al",   ':='(ChargePerDischarge = GrossRevenueInpat / Discharges, CostPerDischarge = CostInpat / Discharges)]
detail_inpat[ Hospital == "Camden Clark"     & FY == 2013 & Payor == "Blue Cross PPO", ':='(ChargePerDischarge = GrossRevenueInpat / Discharges)]
detail_outpat[Hospital == "Pleasant Valley"  & FY == 2013 & Payor == "Aetna Health", DiscountPercOutpat := 22.2]
detail_outpat[Hospital == "St Mary's"        & FY == 2013 & Payor == "Aetna", ChargePerVisit := GrossRevenueOutpat / Visits]

#More rate miscalculations, but with big names
detail_outpat[Hospital == "Princeton Community"            & FY == 2013 & Payor %in% c("Aetna", "Blue Cross"), CostPerVisit := CostOutpat / Visits]
detail_outpat[Hospital == "Charleston Area Medical Center" & FY == 2013 & Payor == "Health Plan of the Upper Ohio Valley", ':='(ChargePerVisit = GrossRevenueOutpat / Visits, CostPerVisit = CostOutpat / Visits)]

#Rate typo 
detail_inpat[ Hospital == "Princeton Community" & FY == 2011 & Payor == "Carelink", ':='(ChargePerDischarge = GrossRevenueInpat  / Discharges, CostPerDischarge = CostInpat  / Discharges)]
detail_outpat[Hospital == "Princeton Community" & FY == 2011 & Payor == "Carelink", ':='(ChargePerVisit     = GrossRevenueOutpat / Visits,     CostPerVisit     = CostOutpat / Visits)]
detail_inpat[ Hospital == "Camden Clark" & FY == 2012 & Payor == "Blue Cross PPO", ChargePerDischarge := 17204.27]
detail_inpat[ Hospital == "Camden Clark" & FY == 2012 & Payor == "Cigna", ChargePerDischarge := 19246.44]

#Apparent duplicate
contract_dt = contract_dt[!Hospital %like% "Parkersburg" | FY != 2007 | Payor != "Multi-Plan"]

#Forgot to write it down 
detail_outpat[Hospital == "Camden Clark" & FY == 2012 & Payor == "Combined Contracts", CostOutpat := 6313712]

#Gets alignment with other elements of net patient revenue  
detail_inpat[ Hospital == "Princeton Community" & FY == 2013 & Payor %in% c("Total", "Aetna"), GrossRevenueInpat := GrossRevenueInpat - 79431]
detail_inpat[ Hospital == "Princeton Community" & FY == 2013 & Payor== "Aetna", ChargePerDischarge := GrossRevenueInpat / Discharges]

#No idea why both have essentially same typo keeping them from cost/charge ratio 
detail_inpat[ Hospital == "Princeton Community" & FY == 2013 & Payor %in% c("Aetna", "Blue Cross"), CostPerDischarge := CostInpat / Discharges]


#Avoids average discount below all but one discount 
detail_inpat[ Hospital == "Princeton Community" & FY == 2013 & Payor %in% c("Total", "Combined Contracts"), GrossRevenueInpat := GrossRevenueInpat - 126491]

#Contract typos (align with DC)
contract_dt[Hospital == "United Hospital Center" & cleanPayor(Payor) == "BCBS-Indemnity/PPO/POS" & FY == 2011,  InpatDiscountPerc := 21.4]
contract_dt[Hospital == "United Hospital Center" & cleanPayor(Payor) == "BCBS-Indemnity/PPO/POS" & FY == 2011, OutpatDiscountPerc := 14]
contract_dt[Hospital == "United Hospital Center" & cleanPayor(Payor) == "BCBS-Indemnity/PPO/POS" & FY == 2012,  InpatDiscountPerc := 13.82]
contract_dt[Hospital == "United Hospital Center" & cleanPayor(Payor) == "BCBS-Indemnity/PPO/POS" & FY == 2012, OutpatDiscountPerc := 20.04]
contract_dt[gsub(" Medical Center", "", Hospital) == "Charleston Area" & FY %in% 2012:2013 & cleanPayor(Payor) == "Health Plan of UOV", OutpatDiscountPerc := 4]
contract_dt[gsub(" Medical Center", "", Hospital) == "Charleston Area" & FY %in% 2012:2013 & cleanPayor(Payor) == "United", OutpatDiscountPerc := 3]
contract_dt[Hospital == "Jackson General" & FY == 2012 & cleanPayor(Payor) %in% paste0("BCBS-", c("Indemnity", "PPO")), InpatDiscountPerc := 58.44]
contract_dt[Hospital == "Reynolds Memorial" & FY == 2012 & cleanPayor(Payor) == "Carelink",  InpatDiscountPerc := 65.61]
contract_dt[Hospital == "Reynolds Memorial" & FY == 2013 & cleanPayor(Payor) == "Carelink", OutpatDiscountPerc := 11.91]
contract_dt[Hospital == "Princeton Community" & FY == 2012 & cleanPayor(Payor) == "BCBS", InpatDiscountPerc := 13.7]
contract_dt[Hospital == "West Virginia University Hospitals" & FY == 2012 & cleanPayor(Payor) == "BCBS Ind &-PPO", OutpatDiscountPerc := 11.82]

contract_dt[Hospital == "Wheeling" & FY == 2013 & cleanPayor(Payor) %in% c("HPUOV", "BCBS-Indemnity"), OutpatDiscountPerc := OutpatDiscountPerc + .01]
contract_dt[Hospital == "Charleston Area"                & FY == 2012 & cleanPayor(Payor) == "HPUOV", OutpatDiscountPerc := 4]
contract_dt[Hospital == "Charleston Area Medical Center" & FY == 2013 & cleanPayor(Payor) == "HPUOV", OutpatDiscountPerc := 4]
contract_dt[Hospital == "Beckley ARH" & FY == 2013 & cleanPayor(Payor) == "BCBS", OutpatDiscountPerc := 15.2]


#Drop single-doctor plans
contract_dt   = contract_dt[  !(Payor %like% "Shultz") & !(Payor %like% "Horacek") & !(Payor %like% "Jirak") & !(Payor %like% "\\(Stern\\)")]
detail_inpat  = detail_inpat[ !(Payor %like% "Shultz") & !(Payor %like% "Horacek") & !(Payor %like% "Jirak") & !(Payor %like% "\\(Stern\\)")]
detail_outpat = detail_outpat[!(Payor %like% "Shultz") & !(Payor %like% "Horacek") & !(Payor %like% "Jirak") & !(Payor %like% "\\(Stern\\)")]

#Combined two contracts
contract_dt   = rbindlist(list(
  contract_dt[   !(Hospital %like% "Charleston Area" & FY == 2011 & Payor == "United OneNet & PPO")],
  copy(contract_dt[Hospital %like% "Charleston Area" & FY == 2011 & Payor == "United OneNet & PPO"])[, Payor := "United OneNet"],
  copy(contract_dt[Hospital %like% "Charleston Area" & FY == 2011 & Payor == "United OneNet & PPO"])[, Payor := "United PPO"]
))

setkeyv(contract_dt, c("Hospital", "Payor", "FY"))

#print(sort(unique(c(contract_dt$Payor, detail_inpat$Payor, detail_outpat$Payor))))

#Rename payors for alignment
contract_dt[,   Payor := cleanPayor(Payor)]
detail_inpat[,  Payor := cleanPayor(Payor)]
detail_outpat[, Payor := cleanPayor(Payor)]


#================================
# Interior-Row Checks
#================================




#I) Annual reports

#CCR check
annual_dt[Hospital == "Davis Medical" & FY == 2016, GrossPatientRev := 224897477]
annual_dt[is.na(CostToChargeRatioInpat), CostToChargeRatioInpat := round(100 * TotalOperatExpense / GrossPatientRev, 2)]
if (nrow(annual_dt[Flag_Unreliable == F & round(100 * TotalOperatExpense / GrossPatientRev - CostToChargeRatioInpat, 2) != 0]) > 0) {
  print("Issue with CCR")
  print(annual_dt[Flag_Unreliable == F & round(100 * TotalOperatExpense / GrossPatientRev - CostToChargeRatioInpat, 2) != 0])
  
  if (!(test_run)) {
    brkhere
  }
}

#Volume threshold check
annual_dt[is.na(VolumeThreshold), VolumeThreshold := round((InpatNongovUtil + OutpatNongovUtil)/20)]
if (nrow(annual_dt[Flag_Unreliable == F & round(VolumeThreshold - (InpatNongovUtil + OutpatNongovUtil)/20) != 0]) > 0) {
  
  print("Issue with volume threshold")
  print(annual_dt[Flag_Unreliable == F & round(VolumeThreshold - (InpatNongovUtil + OutpatNongovUtil)/20) != 0, .(Hospital, FY, VolumeThreshold, PredVT = round((InpatNongovUtil + OutpatNongovUtil)/20), InpatNongovUtil, OutpatNongovUtil)])
  
  if (!(test_run)) {
    brkhere
  }
}


#II) Inpatient detail 



#Net revenue
if (nrow(detail_inpat[Flag_Unreliable == F & abs(GrossRevenueInpat - DiscountAmountInpat - NetRevenueInpat) > 2]) > 0) {
  
  print("Issue with net revenue (Inpatient)")
  print(detail_inpat[Flag_Unreliable == F & abs(GrossRevenueInpat - DiscountAmountInpat - NetRevenueInpat) > 2, .(
    Hospital, FY, Payor, GrossRevenueInpat, DiscountAmountInpat, NetRevenueInpat, Error = (GrossRevenueInpat - DiscountAmountInpat - NetRevenueInpat)
  )])
  
  if (!(test_run)) {
    brkhere
  }
}


#Discount percentage 
detail_inpat[is.na(DiscountPercInpat) & GrossRevenueInpat > 0, DiscountPercInpat := round(100 * DiscountAmountInpat / GrossRevenueInpat, 1)]
if (nrow(detail_inpat[Flag_Unreliable == F & abs(round(100 * DiscountAmountInpat / GrossRevenueInpat, 1) - DiscountPercInpat) > .1]) > 0) {
  
  print("Issue with discount percentage (Inpatient)")
  print(detail_inpat[Flag_Unreliable == F & abs(round(100 * DiscountAmountInpat / GrossRevenueInpat, 1) - DiscountPercInpat) > .1, .(
    Hospital, FY, Payor, GrossRevenueInpat, DiscountPercInpat, PredictedDA = GrossRevenueInpat * DiscountPercInpat / 100, DiscountAmountInpat, Error = abs(DiscountPercInpat * GrossRevenueInpat / (DiscountAmountInpat) - 100)
  )])
  
  if (!(test_run)) {
    brkhere
  }
}


#Charge per discharge 
if (nrow(detail_inpat[Flag_Unreliable == F & abs((GrossRevenueInpat / Discharges - ChargePerDischarge) / ChargePerDischarge) > 0.01 & Discharges > 20]) > 0) {
  
  print("Issue with charge/discharge (Inpatient)")
  print(detail_inpat[Flag_Unreliable == F & abs((GrossRevenueInpat / Discharges - ChargePerDischarge) / ChargePerDischarge) > 0.01 & Discharges > 20, .(
    Hospital, FY, Payor, ChargePerDischarge, ImpliedCPD = GrossRevenueInpat / Discharges, Error = 100 * (GrossRevenueInpat / Discharges - ChargePerDischarge) / ChargePerDischarge
  )])
  
  if (!(test_run)) {
    brkhere
  }
}


#Cost per discharge 
if (nrow(detail_inpat[Flag_Unreliable == F & abs((CostInpat / Discharges - CostPerDischarge) / CostPerDischarge) > 0.01 & Discharges > 20]) > 0) {
  
  print("Issue with cost/discharge (Inpatient)")
  print(detail_inpat[Flag_Unreliable == F & abs((CostInpat / Discharges - CostPerDischarge) / CostPerDischarge) > 0.01 & Discharges > 20, .(
    Hospital, FY, Payor, CostPerDischarge, ImpliedCPD = CostInpat / Discharges, Error = 100 * ((CostInpat / Discharges - CostPerDischarge) / CostPerDischarge)
  )])
  
  if (!(test_run)) {
    brkhere
  }
}



#II) Outpatient detail 



#Net revenue
if (nrow(detail_outpat[Flag_Unreliable == F & abs(GrossRevenueOutpat - DiscountAmountOutpat - NetRevenueOutpat) > 2]) > 0) {
  
  print("Issue with net revenue (Outpatient)")
  print(detail_outpat[Flag_Unreliable == F & abs(GrossRevenueOutpat - DiscountAmountOutpat - NetRevenueOutpat) > 2, .(
    Hospital, FY, Payor, GrossRevenueOutpat, DiscountAmountOutpat, NetRevenueOutpat, Error = (GrossRevenueOutpat - DiscountAmountOutpat - NetRevenueOutpat)
  )])
  
  if (!(test_run)) {
    brkhere
  }
}



#Discount percentage 
detail_outpat[is.na(DiscountPercOutpat) & GrossRevenueOutpat > 0, DiscountPercOutpat := round(100 * DiscountAmountOutpat / GrossRevenueOutpat, 1)]
if (nrow(detail_outpat[Flag_Unreliable == F & abs(round(100 * DiscountAmountOutpat / GrossRevenueOutpat, 1) - DiscountPercOutpat) > .1]) > 0) {
  
  print("Issue with discount percentage (Outpatient)")
  print(detail_outpat[Flag_Unreliable == F & abs(round(100 * DiscountAmountOutpat / GrossRevenueOutpat, 1) - DiscountPercOutpat) > .1, .(
    Hospital, FY, Payor, GrossRevenueOutpat, DiscountPercOutpat, PredictedDA = GrossRevenueOutpat * DiscountPercOutpat / 100, DiscountAmountOutpat, 
    Error = abs(DiscountPercOutpat * GrossRevenueOutpat / (DiscountAmountOutpat) - 100)
  )])
  
  if (!(test_run)) {
    brkhere
  }
}

#Charge per visit 
if (nrow(detail_outpat[abs((GrossRevenueOutpat / Visits - ChargePerVisit) / ChargePerVisit) > 0.01 & abs(GrossRevenueOutpat / Visits - ChargePerVisit) > 5 & Visits > 20]) > 0) {
  
  print("Issue with charge/vist (Outpatient)")
  print(detail_outpat[abs((GrossRevenueOutpat / Visits - ChargePerVisit) / ChargePerVisit) > 0.01 & abs(GrossRevenueOutpat / Visits - ChargePerVisit) > 5 & Visits > 20, .(
    Hospital, FY, Payor, ChargePerVisit, ImpliedCPD = GrossRevenueOutpat / Visits, Error = 100 * (GrossRevenueOutpat / Visits - ChargePerVisit) / ChargePerVisit
  )])
  
  if (!(test_run)) {
    brkhere
  }
}


#Cost per visit
if (nrow(detail_outpat[Flag_Unreliable == F & abs((CostOutpat / Visits - CostPerVisit) / CostPerVisit) > 0.01 & Visits > 20]) > 0) {
  
  print("Issue with cost/visit (Outpatient)")
  print(detail_outpat[Flag_Unreliable == F & abs((CostOutpat / Visits - CostPerVisit) / CostPerVisit) > 0.01 & Visits > 20, .(
    Hospital, FY, Payor, CostPerVisit, ImpliedCPD = CostOutpat / Visits, Error = 100 * ((CostOutpat / Visits - CostPerVisit) / CostPerVisit)
  )])
  
  if (!(test_run)) {
    brkhere
  }
}


#================================
# Interior-Spreadsheet Checks
#================================

if (nrow(contract_dt[, .N, by = .(Hospital, FY, Payor)][N > 1]) > 0) {
  print("Error: Duplicate contracts after cleaning")
  print(contract_dt[, .N, by = .(Hospital, FY, Payor)][N > 1])
  
  if (!(test_run)) {
    brkhere
  }
  
}

if (nrow(contract_dt[is.na(InpatDiscountPerc) & is.na(OutpatDiscountPerc)]) > 0) {
  print("Error: Missing discounts")
  print(contract_dt[is.na(InpatDiscountPerc) & is.na(OutpatDiscountPerc), .(Hospital, FY, Payor)])
  
  
  if (!(test_run)) {
    brkhere
  }
}


#Inpatient totals 
cols = c("Discharges", "GrossRevenueInpat", "DiscountAmountInpat", "NetRevenueInpat", "CostInpat")
inpat_year_errs = rbindlist(lapply(1:length(cols), function(x) {
  col = cols[x]
  detail_inpat[Flag_Unreliable == F, .(Column = col, id = .GRP, col_id = x, NumContracts = .N - 2, Total = sum((Payor == "Total") * get(col), na.rm = TRUE), Pred = sum((Payor != "Total") * get(col), na.rm = TRUE)), by = .(Hospital, FY)]
}))
setkeyv(inpat_year_errs, c("id", "col_id", "Column"))
inpat_year_errs[, c("id", "col_id") := NULL]
inpat_year_errs[, Err := Pred - Total]

if (nrow(inpat_year_errs[abs(Err) > NumContracts + 1]) > 0) {
  
  print("Issue with inpatient contract totals")
  
  print(inpat_year_errs[abs(Err) > NumContracts + 1, .(NumCols = .N), by = .(Hospital, FY)])
  
  if (!(test_run)) {
    brkhere
  }
}


#Outpatient totals 
cols = c("Visits", "GrossRevenueOutpat", "DiscountAmountOutpat", "NetRevenueOutpat", "CostOutpat")
outpat_year_errs = rbindlist(lapply(1:length(cols), function(x) {
  col = cols[x]
  detail_outpat[Flag_Unreliable == F, .(Column = col, id = .GRP, col_id = x, NumContracts = .N - 2, Total = sum((Payor == "Total") * get(col), na.rm = TRUE), Pred = sum((Payor != "Total") * get(col), na.rm = TRUE)), by = .(Hospital, FY)]
}))
setkeyv(outpat_year_errs, c("id", "col_id", "Column"))
outpat_year_errs[, c("id", "col_id") := NULL]
outpat_year_errs[, Err := Pred - Total]

if (nrow(outpat_year_errs[abs(Err) > NumContracts + 1]) > 0) {
  
  print("Issue with outpatient contract totals")
  
  print(outpat_year_errs[abs(Err) > NumContracts + 1, .(NumCols = .N), by = .(Hospital, FY)])
  
  if (!(test_run)) {
    brkhere
  }
}

#================================
# Cross-Spreadsheet Checks
#================================

all_contracts_orig = merge(
  merge(contract_dt, detail_inpat[!Payor %in% c("Combined Contracts", "Total"), .(Hospital, FY, Payor, DiscountPercInpatDt = DiscountPercInpat)], by = c("Hospital", "FY", "Payor"), all.x = TRUE, all.y = TRUE),
  detail_outpat[!Payor %in% c("Combined Contracts", "Total"), .(Hospital, FY, Payor, DiscountPercOutpatDt = DiscountPercOutpat)], by = c("Hospital", "FY", "Payor"), all.x = TRUE, all.y = TRUE
)

key_contracts = all_contracts_orig[!is.na(SeparateFlag) | !is.na(DiscountPercInpatDt) | !is.na(DiscountPercOutpatDt)]

if (nrow(key_contracts[Flag_Unreliable == F & is.na(SeparateFlag)]) > 0) {
  print("Issue with contract match")
  
  print(key_contracts[Flag_Unreliable == F & is.na(SeparateFlag)])
  
  if (!(test_run)) {
    brkhere
  }
}

if (nrow(key_contracts[Flag_Unreliable == F & abs(InpatDiscountPerc - DiscountPercInpatDt) > 0.01]) > 0) {
  
  print("Issue with inpatient contract discount match")
  
  print(key_contracts[Flag_Unreliable == F & abs(InpatDiscountPerc - DiscountPercInpatDt) > 0, .(Hospital, FY, Payor, Flag_Unreliable, DCL = InpatDiscountPerc, DC = DiscountPercInpatDt)])
  
  if (!(test_run)) {
    brkhere
  }
  
}


if (nrow(key_contracts[Flag_Unreliable == F & round(abs(OutpatDiscountPerc - DiscountPercOutpatDt), 5) > 0.01]) > 0) {
  
  print("Issue with outpatient contract discount match")
  
  print(key_contracts[Flag_Unreliable == F & round(abs(OutpatDiscountPerc - DiscountPercOutpatDt), 5) > 0, .(Hospital, FY, Payor, Flag_Unreliable, DCL = OutpatDiscountPerc, DC = DiscountPercOutpatDt)])
  
  if (!(test_run)) {
    brkhere
  }
  
}


#================================
# Output Cleaned Data
#================================

print("Yay! Data cleaning checks passed!")

#Contract table 
all_contracts = merge(
  contract_dt, 
  merge(detail_inpat[!Payor %in% c("Combined Contracts", "Total")], detail_outpat[!Payor %in% c("Combined Contracts", "Total")], by = c("Hospital", "FY", "Payor"), all.x = TRUE, all.y = TRUE),
  by = c("Hospital", "FY", "Payor"), all.x = TRUE, all.y = TRUE
)

#Chance to add missing dates based on DCL 
all_contracts[Date == "" & !is.na(ExpirationDate), Date := ExpirationDate]


#If expired before submission date, change date to Auto 
all_contracts = merge(all_contracts, annual_dt[, .(Hospital, FY, CBMSubmissionDate)], by = c("Hospital", "FY"), all.x = TRUE, all.y = FALSE)

all_contracts[, CountSubmissions := length(unique(!is.na(CBMSubmissionDate))), by = .(Hospital, FY)]
all_contracts[CountSubmissions == 1 & substr(Date, 1, 1) == "2", IsEarly := as.Date(Date) < as.Date(CBMSubmissionDate)]
all_contracts[IsEarly == TRUE, Date := "Auto"]

all_contracts[, c("CBMSubmissionDate, CountSubmissions", "IsEarly") := NULL]

all_contracts[!Date %in% c("", "Auto")]

all_contracts = merge(
  all_contracts,
  merge(
    detail_inpat[ Payor == "Combined Contracts", .(Hospital, FY, DischInpatCombined  = Discharges, GrossRevInpatCombined  = GrossRevenueInpat,  NetRevInpatCombined  = NetRevenueInpat,  CostInpatCombined  = CostInpat)],
    detail_outpat[Payor == "Combined Contracts", .(Hospital, FY, VisitOutpatCombined = Visits,     GrossRevOutpatCombined = GrossRevenueOutpat, NetRevOutpatCombined = NetRevenueOutpat, CostOutpatCombined = CostOutpat)],
    by = c("Hospital", "FY"), all.x = TRUE, all.y = TRUE
  ),
  by = c("Hospital", "FY"), all.x = TRUE, all.y = TRUE
)

all_contracts[, Flag_Unreliable := as.logical(pmax(Flag_Unreliable, Flag_Unreliable.x, Flag_Unreliable.y))]
all_contracts[, c("Flag_Unreliable.x", "Flag_Unreliable.y", "DiscountPercInpat", "DiscountPercOutpat") := NULL]

all_contracts[, Hospital := cleanHospitalName(Hospital)]

all_contracts[, Flag_SpecialCare := Payor %like% "Psych" | Payor %like% "SNF" | Payor %like% "Phys" | Payor %like% "Lab" | gsub("Pro. ", "Pro ", Payor, fixed = T) %like% "Pro Fees"]

fwrite(all_contracts, "Semi-Processed/contract_data.csv")



#Hospital info (including aggregates)
annual_dt = merge(
  merge(
    detail_inpat[ Payor == "Total", .(Hospital, FY, Discharges, ChargesPerDischarge = GrossRevenueInpat  / Discharges, NetRevPerDischarge = NetRevenueInpat  / Discharges, ChargesInpat  = GrossRevenueInpat,  CostInpat)],
    detail_outpat[Payor == "Total", .(Hospital, FY, Visits,     ChargesPerVisit     = GrossRevenueOutpat / Visits,     NetRevPerVisit     = NetRevenueOutpat / Visits,     ChargesOutpat = GrossRevenueOutpat, CostOutpat)],
    by = c("Hospital", "FY"), all.x = TRUE, all.y = TRUE
  ),
  annual_dt,
  by = c("Hospital", "FY"), all.x = TRUE, all.y = TRUE
)

annual_dt[, Flag_Unreliable := NULL]
annual_dt[, Hospital := cleanHospitalName(Hospital)]

fwrite(annual_dt, "Semi-Processed/annual_data.csv")



